from pdfconduit.utils.driver import PdfDriver
from pdfconduit.utils.info import Info
from pdfconduit.utils.path import set_destination, add_suffix
from pdfconduit.utils.read import pypdf_reader
from pdfconduit.utils.receipt import Receipt
from pdfconduit.utils.view import open_window
from pdfconduit.utils.write import write_pdf

__all__ = [
    "set_destination",
    "add_suffix",
    "open_window",
    "write_pdf",
    "Info",
    "Receipt",
    "pypdf_reader",
    "PdfDriver",
]



import inspect
import os
import shutil

test_data_dir = os.path.join(os.path.dirname(__file__), "data")
# pdf_name = 'plan_l.pdf'
# pdf_name = 'plan_p.pdf'
# pdf_name = 'article.pdf'
pdf_name = "document.pdf"
# pdf_name = 'con docs2.pdf'
img_name = "floor plan.png"
pdf_path = os.path.join(test_data_dir, pdf_name)
img_path = os.path.join(test_data_dir, img_name)


def test_data_path(filename):
    return os.path.join(test_data_dir, filename)


def files_are_equal(file1_path, file2_path):
    """
    Compare two files to check if they are the same.

    Parameters:
    file1_path (str): The path to the first file.
    file2_path (str): The path to the second file.

    Returns:
    bool: True if the files are the same, False otherwise.
    """
    with open(file1_path, "rb") as file1, open(file2_path, "rb") as file2:
        file1_content = file1.read()
        file2_content = file2.read()

    return file1_content == file2_content


def function_name_to_file_name(extension=".pdf"):
    return inspect.stack()[1][3] + extension


def copy_pdf_to_output_directory(pdf, save_name):
    shutil.copy(pdf, get_output_filepath(save_name))


def get_output_filepath(filename):
    return os.path.join(os.path.join(os.path.dirname(__file__), "output"), filename)


def expected_equals_output(test_function, output_filepath):
    return files_are_equal(get_output_filepath(test_function), output_filepath)


# Example usage
# Save outputs to tests/output directory
# copy_pdf_to_output_directory({pdfoutput}, function_name_to_file_name())

# Compare saved outputs to generate pdfs
# expected_equals_output(function_name_to_file_name(), {pdfouput})


__all__ = [
    "pdf_path",
    "img_path",
    "test_data_dir",
    "test_data_path",
    "function_name_to_file_name",
    "copy_pdf_to_output_directory",
    "expected_equals_output",
]



# Set directory paths and file names
import os
import sys
from pathlib import Path
from typing import Optional

if "pathlib" in sys.modules:

    def _add_suffix(file_path: str, suffix: str, sep: str, ext: str):
        p = Path(file_path)
        _ext = p.suffix if ext is None else str("." + ext.strip("."))
        out = p.stem + sep + suffix + _ext  # p.suffix is file extension
        return os.path.join(os.path.dirname(file_path), out)

else:

    def _add_suffix(file_path: str, suffix: str, sep: str, ext: str):
        split = os.path.basename(file_path).rsplit(".", 1)
        ext = split[1] if ext is None else str("." + ext.strip("."))
        return os.path.join(
            os.path.dirname(file_path), split[0] + sep + suffix + "." + ext
        )


def add_suffix(
    file_path: str, suffix: str = "modified", sep: str = "_", ext: Optional[str] = None
):
    """Adds suffix to a file name seperated by an underscore and returns file path."""
    return _add_suffix(file_path, suffix, sep, ext)


def set_destination(
    source: str, suffix: str, filename: bool = False, ext: Optional[str] = None
):
    # todo: can be removed, not used
    """Create new pdf filename for temp files"""
    source_dirname = os.path.dirname(source)

    # Do not create nested temp folders (/temp/temp)
    if not source_dirname.endswith("temp"):
        directory = os.path.join(source_dirname, "temp")  # directory
    else:
        directory = source_dirname

    # Create temp dir if it does not exist
    if not os.path.isdir(directory):
        os.mkdir(directory)

    # Parse source filename
    if filename:
        src_file_name = filename
    else:
        src_file_name = Path(source).stem  # file name
    if ext:
        src_file_ext = ext
    else:
        src_file_ext = Path(source).suffix  # file extension

    # Concatenate new filename
    dst_path = src_file_name + "_" + suffix + src_file_ext
    full_path = os.path.join(directory, dst_path)  # new full path

    if not os.path.exists(full_path):
        return full_path
    else:
        # If file exists, increment number until filename is unique
        number = 1
        while True:
            dst_path = src_file_name + "_" + suffix + "_" + str(number) + src_file_ext
            if not os.path.exists(dst_path):
                break
            number = number + 1
        full_path = os.path.join(directory, dst_path)  # new full path
        return full_path



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit import Info, Upscale
from tests import *


class TestUpscale(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path
        cls.temp = TemporaryDirectory()

    @classmethod
    def tearDownClass(cls):
        cls.temp.cleanup()

    @Timer.decorator
    def test_upscale_pdfrw_20x(self):
        s = 2.0
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_2.0_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_upscale_pdfrw_15x(self):
        s = 1.5
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_1.5_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_upscale_pdfrw_30x(self):
        s = 3.0
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_3.0_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_downscale_pdfrw_20x(self):
        s = 1 / 2
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="downscaled_2.0_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_upscale_pypdf_20x(self):
        s = 2.0
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_2.0_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_upscale_pypdf_15x(self):
        s = 1.5
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_1.5_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_upscale_pypdf_30x(self):
        s = 3.0
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="upscaled_3.0_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    @Timer.decorator
    def test_downscale_pypdf_20x(self):
        s = 1 / 2
        upscaled = (
            Upscale(
                pdf_path,
                scale=s,
                suffix="downscaled_2.0_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .upscale()
        )

        self.assertPdfExists(upscaled)
        self.assertPdfUpscaled(s, upscaled)

        expected_equals_output(function_name_to_file_name(), upscaled)

    def assertPdfUpscaled(self, s, upscaled):
        self.assertEqual(
            Info(upscaled).size, tuple([i * s for i in Info(pdf_path).size])
        )
        self.assertEqual(Info(upscaled).pages, Info(pdf_path).pages)

    def assertPdfExists(self, upscaled):
        self.assertTrue(os.path.isfile(upscaled))


if __name__ == "__main__":
    unittest.main()



# Slice PDF to remove unwanted pages
import os
from tempfile import NamedTemporaryFile
from typing import Optional

from pypdf import (
    PdfReader as pypdfReader,
    PdfWriter as pypdfWriter,
)

from pdfconduit.utils.info import Info
from pdfconduit.utils.path import add_suffix


def slicer(
    document: str,
    first_page: Optional[int] = None,
    last_page: Optional[int] = None,
    suffix: str = "sliced",
    tempdir: Optional[str] = None,
):
    # Set output file name
    if tempdir:
        with NamedTemporaryFile(suffix=".pdf", dir=tempdir, delete=False) as temp:
            output = temp.name
    elif suffix:
        output = os.path.join(os.path.dirname(document), add_suffix(document, suffix))
    else:
        with NamedTemporaryFile(suffix=".pdf") as temp:
            output = temp.name

    # Reindex page selections for simple user input
    first_page = first_page - 1 if not None else None

    # Validate page range by comparing selection to number of pages in PDF document
    pages = Info(document).pages
    invalid = (
        "Number of pages: "
        + str(pages)
        + " ----> Page Range Input: "
        + str(first_page)
        + "-"
        + str(last_page)
    )
    assert first_page <= last_page <= pages, invalid

    reader = pypdfReader(document)
    writer = pypdfWriter()

    for page_num in list(range(reader.get_num_pages()))[first_page:last_page]:
        writer.add_page(reader.pages[page_num])

    with open(output, "wb") as fp:
        writer.write(fp)

    return output



from pdfconduit.modify.canvas.constructor import CanvasConstructor
from pdfconduit.modify.canvas.objects import CanvasImg, CanvasStr, CanvasObjects

__all__ = ["CanvasConstructor", "CanvasImg", "CanvasStr", "CanvasObjects"]



import unittest

from looptools import Timer

from pdfconduit import Info
from tests import *


class TestInfo(unittest.TestCase):
    @Timer.decorator
    def test_is_encrypted(self):
        info = self._get_info("encrypted.pdf")

        self.assertTrue(info.encrypted)

    @Timer.decorator
    def test_is_decrypted(self):
        info = self._get_info("article.pdf")

        self.assertTrue(info.decrypted)

    @Timer.decorator
    def test_is_not_encrypted(self):
        info = self._get_info("article.pdf")

        self.assertFalse(info.encrypted)

    @Timer.decorator
    def test_is_not_decrypted(self):
        info = self._get_info("encrypted.pdf")

        self.assertFalse(info.decrypted)

    @Timer.decorator
    def test_pages(self):
        info = self._get_info("article.pdf")

        self.assertIsInstance(info.pages, int)
        self.assertEqual(info.pages, 1)
        self.assertEqual(Info(test_data_path("document.pdf")).pages, 11)

    @Timer.decorator
    def test_metadata(self):
        info = self._get_info("article.pdf")

        self.assertIsInstance(info.metadata, dict)
        self.assertEqual(
            info.metadata["/Creator"], "This PDF is created by PDF4U Pro 2.0"
        )
        self.assertEqual(
            info.metadata["/CreationDate"],
            "D:20040120105826",
        )
        self.assertEqual(
            info.metadata["/Producer"],
            "PDF4U Adobe PDF Creator 2.0",
        )

    @Timer.decorator
    def test_resources(self):
        info = self._get_info("article.pdf")

        self.assertEqual(info.pages, len(info.resources()))

        resources = info.resources()[0]
        self.assertIsInstance(resources, dict)
        self.assertEqual(resources["/Type"], "/Page")

        mediabox = list(resources["/MediaBox"])
        self.assertEqual(mediabox[0], 0)
        self.assertEqual(mediabox[1], 0)
        self.assertEqual(float(mediabox[2]), 595.276)
        self.assertEqual(float(mediabox[3]), 841.89)

    @Timer.decorator
    def test_security_encrypted_pdf(self):
        info = self._get_info("encrypted.pdf")

        self.assertIsInstance(info.security, dict)
        self.assertTrue("/V" in info.security)
        self.assertTrue("/R" in info.security)
        self.assertTrue("/Length" in info.security)
        self.assertTrue("/P" in info.security)
        self.assertTrue("/Filter" in info.security)
        self.assertTrue("/O" in info.security)
        self.assertTrue("/U" in info.security)

    @Timer.decorator
    def test_security_decrypted_pdf(self):
        info = self._get_info("encrypted.pdf", "foo")

        self.assertIsInstance(info.security, dict)
        self.assertTrue("/V" in info.security)
        self.assertTrue("/R" in info.security)
        self.assertTrue("/Length" in info.security)
        self.assertTrue("/P" in info.security)
        self.assertTrue("/Filter" in info.security)
        self.assertTrue("/O" in info.security)
        self.assertTrue("/U" in info.security)

    @Timer.decorator
    def test_security_passwordless_pdf(self):
        info = self._get_info("article.pdf")

        self.assertIsInstance(info.security, dict)
        self.assertEqual(info.security, {})

    @Timer.decorator
    def test_dimensions(self):
        info = self._get_info("article.pdf")

        self.assertIsInstance(info.dimensions, dict)
        self.assertTrue("w" in info.dimensions)
        self.assertTrue("h" in info.dimensions)
        self.assertIsInstance(info.dimensions["w"], float)
        self.assertIsInstance(info.dimensions["h"], float)
        self.assertEqual(info.dimensions["w"], 595.276)
        self.assertEqual(info.dimensions["h"], 841.89)

    @Timer.decorator
    def test_size(self):
        info = self._get_info("article.pdf")

        self.assertIsInstance(info.size, tuple)
        self.assertEqual(len(info.size), 2)
        self.assertIsInstance(info.size[0], float)
        self.assertIsInstance(info.size[1], float)
        self.assertEqual(info.size[0], 595.276)
        self.assertEqual(info.size[1], 841.89)

    @Timer.decorator
    def test_size_and_dimensions_are_equal(self):
        info = self._get_info("article.pdf")

        self.assertEqual(info.size[0], info.dimensions["w"])
        self.assertEqual(info.size[1], info.dimensions["h"])

    @Timer.decorator
    def test_rotate_no_rotation(self):
        info = self._get_info("article.pdf")

        self.assertEqual(info.rotate, None)

    @Timer.decorator
    def test_rotate_rotated(self):
        info = self._get_info("rotated.pdf")

        self.assertIsInstance(info.rotate, int)
        self.assertEqual(info.rotate, 90)

    @staticmethod
    def _get_info(filename, password=None):
        return Info(test_data_path(filename), password=password)


if __name__ == "__main__":
    unittest.main()



from pdfconduit.transform.merge import Merge
from pdfconduit.transform.rotate import Rotate
from pdfconduit.transform.slice import slicer
from pdfconduit.transform.upscale import Upscale

__all__ = ["Merge", "Rotate", "slicer", "Upscale"]



# Rotate a pdf file
import os
from tempfile import NamedTemporaryFile
from typing import Optional

from pdfrw import (
    PdfReader as PdfrwReader,
    PdfWriter as PdfrwWriter,
)
from pypdf import PdfReader as PypdfReader, PdfWriter as PypdfWriter

from pdfconduit.utils.driver import PdfDriver
from pdfconduit.utils.path import add_suffix


class Rotate(PdfDriver):

    def __init__(
        self,
        file_name: str,
        rotation: int,
        suffix: str = "rotated",
        tempdir: Optional[str] = None,
    ):
        self.file_name = file_name
        self.rotation = rotation
        self.suffix = suffix
        self.tempdir = tempdir

        if tempdir:
            with NamedTemporaryFile(suffix=".pdf", dir=tempdir, delete=False) as temp:
                self.outfn = temp.name
        elif suffix:
            self.outfn = os.path.join(
                os.path.dirname(file_name), add_suffix(file_name, suffix)
            )
        else:
            self.outfn = NamedTemporaryFile(suffix=".pdf").name

    def __str__(self) -> str:
        return self.file

    def rotate(self) -> str:
        return self.execute()

    @property
    def file(self) -> str:
        return str(self.outfn)

    def pdfrw(self) -> str:
        trailer = PdfrwReader(self.file_name)
        pages = trailer.pages

        ranges = [[1, len(pages)]]

        for onerange in ranges:
            onerange = (onerange + onerange[-1:])[:2]
            for pagenum in range(onerange[0] - 1, onerange[1]):
                pages[pagenum].Rotate = (
                    int(pages[pagenum].inheritable.Rotate or 0) + self.rotation
                ) % 360

        outdata = PdfrwWriter(self.outfn)
        outdata.trailer = trailer
        outdata.write()
        return self.outfn

    def pypdf(self) -> str:
        reader = PypdfReader(self.file_name)
        writer = PypdfWriter()

        for page_num in range(1, reader.get_num_pages()):
            writer.add_page(reader.pages[page_num]).rotate(self.rotation)

        with open(self.outfn, "wb") as fp:
            writer.write(fp)

        return self.outfn


def rotate(
    file_name: str,
    rotation: int,
    suffix: str = "rotated",
    tempdir: Optional[str] = None,
    method: str = "pdfrw",
):
    """Rotate PDF by increments of 90 degrees."""
    # todo: remove use of function or clean this up
    rotater = Rotate(file_name, rotation, suffix, tempdir)
    if method == "pdfrw":
        rotater.use_pdfrw()
    else:
        rotater.use_pypdf()
    return rotater.rotate()



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit import Info, slicer
from tests import *


class TestSlice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_slice_pypdf_only_first_page(self):
        """Slice a page range from a PDF to create a new 'trimmed' pdf file."""
        fp = 1
        lp = 1
        sliced = slicer(
            self.pdf_path,
            first_page=fp,
            last_page=lp,
            tempdir=self.temp.name,
        )

        self.assertPdfExists(sliced)
        self.assertCorrectPagesSliced(fp, lp, sliced)

        expected_equals_output(function_name_to_file_name(), sliced)

    @Timer.decorator
    def test_slice_pypdf_4th_through_7th_pages(self):
        """Slice a page range from a PDF to create a new 'trimmed' pdf file."""
        fp = 4
        lp = 7
        sliced = slicer(
            self.pdf_path,
            first_page=fp,
            last_page=lp,
            tempdir=self.temp.name,
        )

        self.assertPdfExists(sliced)
        self.assertCorrectPagesSliced(fp, lp, sliced)

        expected_equals_output(function_name_to_file_name(), sliced)

    def assertCorrectPagesSliced(self, fp, lp, sliced):
        self.assertEqual(Info(sliced).pages, len(range(fp, lp + 1)))

    def assertPdfExists(self, sliced):
        self.assertTrue(os.path.isfile(sliced))


if __name__ == "__main__":
    unittest.main()



import os
from datetime import datetime
from typing import Any

try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class Receipt:
    # todo: refactor to write to logs
    def __init__(self, use: bool = True):
        self.dst = None
        self.use = use
        self.items = []
        self._print = print

        self.add("PDF Watermarker", datetime.now().strftime("%Y-%m-%d %H:%M"))

    def set_dst(self, doc: str, file_name: str = "watermark receipt.txt") -> Self:
        self.dst = os.path.join(os.path.dirname(doc), file_name)
        self.add("Directory", os.path.dirname(doc))
        self.add("PDF", os.path.basename(doc))
        return self

    def add(self, key: str, value: Any):
        message = str("{0:20}--> {1}".format(key, value))
        if self.use:
            self._print(message)
        self.items.append(message)

    def dump(self):
        exists = os.path.isfile(self.dst)
        with open(self.dst, "a") as f:
            if exists:
                f.write(
                    "*******************************************************************\n"
                )

            for item in self.items:
                f.write(item + "\n")

        print("Success!")



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit.conduit import Watermark
from pdfconduit.conduit.watermark.label import Label
from pdfconduit.utils import Info
from tests import *


class TestWatermark(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()
        self.address = "43 Indian Lane"
        self.town = "Franklin"
        self.state = "MA"
        self.rotate = 30
        self.owner_pw = "foo"
        self.user_pw = "baz"

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_conduit_watermark_pdfrw(self):
        """Apply a watermark to all pages of PDF using the `pdfrw` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
            flatten=False,
        )
        added = w.add(self.pdf_path, wtrmrk, method="pdfrw", suffix=None)

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_underneath_pdfrw(self):
        """Apply a watermark underneath original content of PDF using the `pdfrw` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
        )
        added = w.add(
            self.pdf_path, wtrmrk, underneath=True, suffix=None, method="pdfrw"
        )

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_overlay_pdfrw(self):
        """Apply a watermark overlaid over original content of PDF using the `pdfrw` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
        )
        added = w.add(
            self.pdf_path, wtrmrk, underneath=False, suffix=None, method="pdfrw"
        )

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_flat_pdfrw(self):
        """Apply a flattened watermark to a PDF using the `pdfrw` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        flat = w.draw(
            self.address, str(self.town + ", " + self.state), opacity=0.08, flatten=True
        )
        added = w.add(self.pdf_path, flat, suffix=None, method="pdfrw")

        self.assertPdfExists(flat)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_layered_pdfrw(self):
        """Apply a flattened watermark to a PDF using the `pdfrw` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        layered = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            flatten=False,
        )
        added = w.add(self.pdf_path, layered, suffix=None, method="pdfrw")

        # Assert the watermark file exists
        self.assertPdfExists(layered)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_pypdf(self):
        """Apply a watermark to all pages of PDF using the `pypdf` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
            flatten=False,
        )
        added = w.add(self.pdf_path, wtrmrk, method="pypdf", suffix=None)

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_underneath_pypdf(self):
        """Apply a watermark underneath original content of PDF using the `pypdf` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
        )
        added = w.add(
            self.pdf_path, wtrmrk, underneath=True, suffix=None, method="pypdf"
        )

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_overlay_pypdf(self):
        """Apply a watermark overlaid over original content of PDF using the `pypdf` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        wtrmrk = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
        )
        added = w.add(
            self.pdf_path, wtrmrk, underneath=False, suffix=None, method="pypdf"
        )

        self.assertPdfExists(wtrmrk)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_flat_pypdf(self):
        """Apply a flattened watermark to a PDF using the `pypdf` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        flat = w.draw(
            self.address, str(self.town + ", " + self.state), opacity=0.08, flatten=True
        )
        added = w.add(self.pdf_path, flat, suffix=None, method="pypdf")

        self.assertPdfExists(flat)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_layered_pypdf(self):
        """Apply a flattened watermark to a PDF using the `pypdf` method."""
        w = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        layered = w.draw(
            self.address,
            str(self.town + ", " + self.state),
            opacity=0.08,
            flatten=False,
        )
        added = w.add(self.pdf_path, layered, suffix=None, method="pypdf")

        # Assert the watermark file exists
        self.assertPdfExists(layered)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

        expected_equals_output(function_name_to_file_name(), added)

    @Timer.decorator
    def test_conduit_watermark_label(self):
        """Apply a watermark label to a PDF file."""
        label = os.path.basename(self.pdf_path)
        labeled = Label(
            self.pdf_path, label, tempdir=self.temp.name, suffix=None
        ).write(cleanup=False)

        self.assertPdfExists(labeled)
        self.assertPdfHasResources(labeled)

        expected_equals_output(function_name_to_file_name(), labeled)

    def assertPdfExists(self, pdf):
        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(pdf))

    def assertPdfHasResources(self, pdf):
        # Assert watermarked PDF has page resources
        self.assertTrue(Info(pdf).resources())


if __name__ == "__main__":
    unittest.main()



import unittest

from looptools import Timer
from pypdf import PdfReader
from pypdf.errors import PdfReadError

from pdfconduit.utils import pypdf_reader
from tests import *


class TestRead(unittest.TestCase):
    @Timer.decorator
    def test_pypdf_reader_can_read_unencrypted(self):
        file_path = test_data_path("document.pdf")
        reader = pypdf_reader(file_path)

        self.assertIsInstance(reader, PdfReader)
        self.assertEqual(reader.get_num_pages(), 11)

    @Timer.decorator
    def test_pypdf_reader_cant_read_encrypted(self):
        file_path = test_data_path("encrypted.pdf")
        reader = pypdf_reader(file_path)

        with self.assertRaises(PdfReadError) as context:
            reader.get_num_pages()

        self.assertIsInstance(context.exception, PdfReadError)
        self.assertEqual("File has not been decrypted", context.exception.__str__())

    @Timer.decorator
    def test_pypdf_reader_can_read_encrypted_with_password(self):
        file_path = test_data_path("encrypted.pdf")
        reader = pypdf_reader(file_path, "foo")

        self.assertIsInstance(reader, PdfReader)
        self.assertTrue(reader.is_encrypted)
        self.assertEqual(reader.get_num_pages(), 11)


if __name__ == "__main__":
    unittest.main()



from pdfconduit.conduit import Encrypt, Watermark
from pdfconduit.convert import IMG2PDF, PDF2IMG, Flatten
from pdfconduit.transform import Merge, Rotate, Upscale, slicer
from pdfconduit.utils import Info

__all__ = [
    "Encrypt",
    "Watermark",
    "IMG2PDF",
    "PDF2IMG",
    "Flatten",
    "Merge",
    "Rotate",
    "Upscale",
    "slicer",
    "Info",
]



import os
import unittest
from tempfile import NamedTemporaryFile

from looptools import Timer

from pdfconduit.conduit import Encrypt
from pdfconduit.utils import Info
from tests import *


class TestEncrypt(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Encryption passwords
        cls.owner_pw = "foo"
        cls.user_pw = "baz"
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = NamedTemporaryFile(suffix=".pdf", delete=False)

    def tearDown(self):
        if os.path.exists(self.temp.name):
            os.remove(self.temp.name)

    @Timer.decorator
    def test_encrypt_128bit(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=True,
            suffix="128bit",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert128BitEncryption(security)
        self.assertSecurityValue(security, 4)
        self.assertPermissions(encrypted, can_print=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_40bit(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=False,
            suffix="40bit",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert40BitEncryption(security)
        self.assertSecurityValue(security, 4)
        self.assertPermissions(encrypted, can_print=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_128bit_allow_printing(self):
        # todo: allow and don't allow tests
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            allow_printing=True,
            bit128=True,
            output=self.temp.name,
            suffix="128bit_allow_printing",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert128BitEncryption(security)
        self.assertSecurityValue(security, 4)
        self.assertPermissions(encrypted, can_print=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_128bit_allow_commenting(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            allow_printing=False,
            allow_commenting=True,
            bit128=True,
            suffix="128bit_allow_commenting",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assertSecurityValue(security, 8)
        self.assertPermissions(encrypted, can_print=False, can_modify=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_128bit_allow_printing_and_commenting(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            allow_printing=True,
            allow_commenting=True,
            bit128=True,
            suffix="128bit_allow_printing_and_commenting",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert128BitEncryption(security)
        self.assertSecurityValue(security, 12)
        self.assertPermissions(encrypted, can_print=True, can_modify=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_40bit_allow_printing(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            allow_printing=True,
            bit128=False,
            output=self.temp.name,
            suffix="40bit_allow_printing",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert40BitEncryption(security)
        self.assertSecurityValue(security, 4)
        self.assertPermissions(encrypted, can_print=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_40bit_allow_commenting(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            allow_printing=False,
            allow_commenting=True,
            bit128=False,
            suffix="40bit_allow_commenting",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert40BitEncryption(security)
        self.assertSecurityValue(security, 8)
        self.assertPermissions(encrypted, can_print=False, can_modify=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypt_40bit_allow_printing_and_commenting(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            allow_printing=True,
            allow_commenting=True,
            bit128=False,
            suffix="40bit_allow_printing_and_commenting",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert40BitEncryption(security)
        self.assertSecurityValue(security, 12)
        self.assertPermissions(encrypted, can_print=True, can_modify=True)

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_password_byte_string(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            suffix="byte_string",
        )

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert128BitEncryption(security)
        self.assertSecurityValue(security, 4)

        self.assertPermissions(encrypted, can_print=True)

        # Assert user & owner password byte string is correct
        self.assertEqual(
            security["/O"],
            "ÐHÑRžS]˘—Í8V6{˘KJ’ß\x01æ7ÑnÊ\x06[¯Nd’\x0b",
        )

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    @Timer.decorator
    def test_encrypted_pdf_has_metadata(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=True,
            suffix="metadata",
        )

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)

        metadata = Info(encrypted.output, self.user_pw).metadata
        self.assertEqual(metadata["/Producer"], "pdfconduit")
        self.assertEqual(metadata["/Creator"], "pdfconduit")
        self.assertEqual(metadata["/Author"], "Stephen Neal")

        expected_equals_output(function_name_to_file_name(), encrypted.output)

    def _getPdfSecurity(self, encrypted):
        return Info(encrypted.output, self.user_pw).security

    def assertPdfExists(self, encrypted):
        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

    def assertEncrypted(self, pdf):
        self.assertTrue(Info(pdf.output, self.user_pw).encrypted)

    def assert128BitEncryption(self, security):
        self.assertTrue("/Length" in security)
        self.assertIsInstance(security["/Length"], int)
        self.assertEqual(security["/Length"], 128)

        self.assertEqual(security["/Filter"], "/Standard")

        # Assert standard security handler revision is 3
        self.assertEqual(security["/R"], 3)

        # Assert encryption algo code is 2 (3.1 algo with key length of 40 to 128bits)
        self.assertEqual(security["/R"], 3)

    def assert40BitEncryption(self, security):
        if "/Length" in security:
            self.assertEqual(security["/Length"], 40)

        # Assert standard security handler revision is 2
        self.assertEqual(security["/R"], 2)

        # Assert encryption algo code is 1 (3.1 algo with key length of 40bits)
        self.assertEqual(security["/R"], 2)

    def assertSecurityValue(self, security, expected):
        self.assertTrue("/P" in security)
        self.assertEqual(security["/P"], expected)

    def assertPermissions(
        self,
        pdf,
        can_print=False,
        can_modify=False,
        can_copy=False,
        can_annotate=False,
        can_fill_forms=False,
        can_change_accessability=False,
        can_assemble=False,
        can_print_high_quality=False,
    ):
        permissions = Info(pdf.output, self.user_pw).permissions
        self.assertEqual(permissions.can_print(), can_print)
        self.assertEqual(permissions.can_modify(), can_modify)
        self.assertEqual(permissions.can_copy(), can_copy)
        self.assertEqual(permissions.can_annotate(), can_annotate)
        self.assertEqual(permissions.can_fill_forms(), can_fill_forms)
        self.assertEqual(
            permissions.can_change_accessability(), can_change_accessability
        )
        self.assertEqual(permissions.can_assemble(), can_assemble)
        self.assertEqual(permissions.can_print_high_quality(), can_print_high_quality)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit import Info, Rotate
from tests import *


class TestRotate(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_rotate_pdfrw_90(self):
        """Rotate a PDF file by 90 degrees using the `pdfrw` library."""
        rotation = 90
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_90_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    @Timer.decorator
    def test_rotate_pdfrw_180(self):
        """Rotate a PDF file by 180 degrees using the `pdfrw` library."""
        rotation = 180
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_180_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    @Timer.decorator
    def test_rotate_pdfrw_270(self):
        """Rotate a PDF file by 270 degrees using the `pdfrw` library."""
        rotation = 270
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_270_pdfrw",
                tempdir=self.temp.name,
            )
            .use_pdfrw()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    @Timer.decorator
    def test_rotate_pypdf_90(self):
        """Rotate a PDF file by 90 degrees using the `pypdf` library."""
        rotation = 90
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_90_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    @Timer.decorator
    def test_rotate_pypdf_180(self):
        """Rotate a PDF file by 180 degrees using the `pypdf` library."""
        rotation = 180
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_180_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    @Timer.decorator
    def test_rotate_pypdf_270(self):
        """Rotate a PDF file by 270 degrees using the `pypdf` library."""
        rotation = 270
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_270_pypdf",
                tempdir=self.temp.name,
            )
            .use_pypdf()
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

        expected_equals_output(function_name_to_file_name(), rotated)

    def assertPdfExists(self, pdf):
        # Assert rotated pdf file exists
        self.assertTrue(os.path.isfile(pdf))

    def assertPdfRotation(self, rotated, rotation):
        self.assertEqual(Info(rotated).rotate, rotation)


if __name__ == "__main__":
    unittest.main()



import os.path
import unittest

from looptools import Timer

from pdfconduit.utils import add_suffix
from tests import *


class TestPath(unittest.TestCase):
    @Timer.decorator
    def test_add_suffix(self):
        file_path = test_data_path("article.pdf")
        with_suffix = add_suffix(file_path)

        self.assertIsInstance(with_suffix, str)
        self.assertEqual(
            with_suffix,
            os.path.join(os.path.dirname(file_path), "article_modified.pdf"),
        )

    @Timer.decorator
    def test_add_suffix_suffix(self):
        file_path = test_data_path("article.pdf")
        with_suffix = add_suffix(file_path, "new")

        self.assertIsInstance(with_suffix, str)
        self.assertEqual(
            with_suffix, os.path.join(os.path.dirname(file_path), "article_new.pdf")
        )

    @Timer.decorator
    def test_add_suffix_suffix_sep(self):
        file_path = test_data_path("article.pdf")
        with_suffix = add_suffix(file_path, "old", "-")

        self.assertIsInstance(with_suffix, str)
        self.assertEqual(
            with_suffix, os.path.join(os.path.dirname(file_path), "article-old.pdf")
        )

    @Timer.decorator
    def test_add_suffix_suffix_sep_ext(self):
        file_path = test_data_path("article.pdf")
        with_suffix = add_suffix(file_path, "backup", "-", "zip")

        self.assertIsInstance(with_suffix, str)
        self.assertEqual(
            with_suffix, os.path.join(os.path.dirname(file_path), "article-backup.zip")
        )


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory

from looptools import Timer

from pdfconduit import Info, Merge
from tests import *


class TestMerge(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdfs = [
            os.path.join(test_data_dir, p)
            for p in ["article.pdf", "charts.pdf", "document.pdf", "manual.pdf"]
        ]

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @Timer.decorator
    def test_merge_pdfrw(self):
        """Merge multiple PDF files into a single PDF using the `pdfrw` library."""
        merged = Merge(
            self.pdfs,
            output_name="merged_pdfrw",
            output_dir=self.temp.name,
        ).use_pdfrw()
        merged.merge()

        # Assert merged file exists
        self.assertTrue(os.path.exists(merged.file))

        # Assert sum of pages in original pdf files equals sum of pages in merged pdf
        self.assertEqual(
            sum([Info(pdf).pages for pdf in self.pdfs]), Info(merged.file).pages
        )

        # Assert metadata was added correctly
        metadata = Info(merged.output).metadata
        self.assertEqual(metadata["/Producer"], "pdfconduit")
        self.assertEqual(metadata["/Creator"], "pdfconduit")
        self.assertEqual(metadata["/Author"], "Stephen Neal")

        expected_equals_output(function_name_to_file_name(), merged.output)

    @Timer.decorator
    def test_merge_pypdf(self):
        """Merge multiple PDF files into a single PDF using the `pypdf` library."""
        merged = Merge(
            self.pdfs,
            output_name="merged_pypdf",
            output_dir=self.temp.name,
        ).use_pypdf()
        merged.merge()

        # Assert merged file exists
        self.assertTrue(os.path.exists(merged.file))

        # Assert sum of pages in original pdf files equals sum of pages in merged pdf
        self.assertEqual(
            sum([Info(pdf).pages for pdf in self.pdfs]), Info(merged.file).pages
        )

        # Assert metadata was added correctly
        metadata = Info(merged.output).metadata
        self.assertEqual(metadata["/Producer"], "pypdf")

        expected_equals_output(function_name_to_file_name(), merged.output)


if __name__ == "__main__":
    unittest.main()



# Encrypt a PDF file with password protection
from typing import Optional

from pypdf import PdfWriter
from pypdf.constants import UserAccessPermissions

from pdfconduit.utils import add_suffix
from pdfconduit.utils.read import pypdf_reader


class Encrypt:
    def __init__(
        self,
        pdf: str,
        user_pw: str,
        owner_pw: Optional[str] = None,
        output: Optional[str] = None,
        suffix: str = "secured",
        bit128: bool = True,
        allow_printing: bool = True,
        allow_commenting: bool = False,
        overwrite_permission: Optional[int] = None,
        decrypt: Optional[str] = None,
    ) -> None:
        """Password protect PDF file and allow all other permissions."""
        self.pdf = pdf
        self.user_pw = user_pw
        self.owner_pw = owner_pw
        self.output = add_suffix(pdf, suffix=suffix) if not output else output
        self.encrypt_128 = bit128
        self.allow_printing = allow_printing
        self.allow_commenting = allow_commenting
        self.overwrite_permission = overwrite_permission

        # todo: add algorythm parameter

        self.encrypt(decrypt)

    def __str__(self) -> str:
        return str(self.output)

    def encrypt(self, decrypt: Optional[str] = None) -> str:
        if self.allow_printing and self.allow_commenting:
            permissions = UserAccessPermissions.PRINT | UserAccessPermissions.MODIFY
        elif self.allow_printing:
            permissions = UserAccessPermissions.PRINT
        elif self.allow_commenting:
            permissions = UserAccessPermissions.MODIFY
        else:
            permissions = UserAccessPermissions

        with open(self.pdf, "rb") as pdf_file:
            # Read opened PDF file
            pdf_reader = pypdf_reader(pdf_file, decrypt)

            # Create PDF writer object
            pdf_writer = PdfWriter(clone_from=pdf_reader)

            # Apply encryption to writer object
            pdf_writer.encrypt(
                user_password=self.user_pw,
                owner_password=self.owner_pw,
                use_128bit=self.encrypt_128,
                permissions_flag=permissions,
            )

            # todo: add metadata adding functionality
            pdf_writer.add_metadata(
                {
                    "/Producer": "pdfconduit",
                    "/Creator": "pdfconduit",
                    "/Author": "Stephen Neal",
                }
            )

            # Write encrypted PDF to file
            with open(self.output, "wb") as output_pdf:
                pdf_writer.write(output_pdf)
        return self.output


def main():
    from argparse import ArgumentParser

    # TODO: Finish implementing argparse

    # Declare argparse argument descriptions
    usage = "Encrypt PDF files"
    description = "Password protect a PDF file by applying encryption."
    helpers = {
        "pdf": "PDF file to encrypt",
        "p": "Retrieve Python interpreter information such as Python version and Python compiler",
        "s": "Retrieve software related information such as operating system, machine type, username, etc.",
        "hw": "Retrieve Hardware related information such as memory and processor information",
    }

    # construct the argument parse and parse the arguments
    ap = ArgumentParser(usage=usage, description=description)
    ap.add_argument("pdf", help=helpers["pdf"])
    ap.add_argument("-upw", "--user_pw", help=helpers["upw"])
    ap.add_argument("-opw", "--owner_pw", help=helpers["opw"])
    ap.add_argument("-out", "--output", help=helpers["out"])
    args = vars(ap.parse_args())


if __name__ == "__main__":
    main()
